-- A. Skeletal Entity
function newEntity(x, y, width, height)

  local entity = {
    -- Coordinates
    x = x or 0,
    y = y or 0,
    -- Rotation
    r = 0,
    -- Scale
    sx = 1,
    sy = 1,
    -- Dimensions and offsets(For Rectangle Drawings)
    width = width or 1,
    height = height or 1,
  }
  entity.ox = entity.width / 2
  entity.oy = entity.height / 2
  
  function entity:draw()
    love.graphics.rectangle('line', self.x - self.ox, self.y - self.oy, self.width, self.height)
  end
  
  -- Transform Functions (Absolute)
  function entity:setPosition(x, y)
    self.x = x or self.x
    self.y = y or self.y
  end
  function entity:setRotation(r)
    self.r = math.rad(r)
  end
  function entity:setScale(sx, sy)
    self.sx = sx or 1
    self.sy = sy or sx
  end
  
  -- Transform Functions (Relative)
  function entity:move(dx, dy)
    self.x = self.x + (dx or 0)
    self.y = self.y + (dy or 0)
  end
  function entity:rotate(dr)
    self.r = self.r + math.rad(dr) or 0
  end
  function entity:scale(sx, sy)
    self.sx = self.sx + (sx or 1)
    self.sy = self.sy + (sy or sx)
  end
  
  return entity
end

-- B. Static Entity Kit
function newStaticEntityKit(image)
  local kit = {}
  
  local quads = {}
  
  function kit.newQuad(id, x, y, w, h)
    quads[id] = love.graphics.newQuad(x, y, w, h, image:getWidth(), image:getHeight())
  end
  
  function kit.spawn(x, y, id)
    local entity = newEntity(x, y)
    
    if id then
      _, _, entity.width, entity.height = quads[id]:getViewport()
      
      function entity:draw()
        love.graphics.draw(image, quads[id], self.x, self.y, self.r,
          self.sx, self.sy, self.ox, self.oy
        )
      end
      
    else
      entity.width, entity.height = image:getDimensions()
      
      function entity:draw()
        love.graphics.draw(image, self.x, self.y, self.r,
          self.sx, self.sy, self.ox, self.oy
        )
      end
    end
    
    entity.ox, entity.oy = entity.width / 2, entity.height / 2
    
    return entity
  end
  
  return kit
end

-- C. Animated Entity Kit
function newAnimatedEntityKit(image)
  local kit = {}
  
  local animations = {} -- The kit has internal animations
  
  -- C.1. Animation Generator
  function kit.newAnimation(id, framevars, frame_rate)
    -- a. Generating a set of Quads/Frames
    local x         = framevars[1]
    local y         = framevars[2]
    local width     = framevars[3]
    local height    = framevars[4]
    local intervals = framevars[5]

    local frames = {}
    for i = 0, intervals - 1 do
      local start_x = x + ((width + 1) * i)
      local quad = love.graphics.newQuad(start_x, y, width, height, image:getWidth(), image:getHeight())
      table.insert(frames, quad)
    end
    
    -- b. Putting the Frames and the Frame_rate into an animation
    animations[id] = { frames = frames, frame_rate = frame_rate}
  end
  
  -- Just in case you want the entity to be dependent on the animation's dimensions.
  function kit.getDimensions(id)
    local _, _, w, h = animations[id].frames[1]:getViewport()
    return w, h, w/2, h/2
  end
  
  -- C.2. Animated Entity Generator
  function kit.spawn(x, y, width, height, animation)
    local entity = newEntity(x, y, width, height)
    
    -- Animated Entity Basic Requirements
    local current_animation = animation
    local current_frame = 1
    local frame_timer = 0
    
    entity.playing = true
    local looping = true
    
    if animation then
      _, _, entity.ox, entity.oy = kit.getDimensions(animation)
    end

    function entity:replayAnimation()
      current_frame = 1
      frame_timer = 0
      self.playing = true
    end
    
    function entity:setAnimation(id, loop)
      -- Resetting values if moving to another animation
      if current_animation ~= id then
        current_animation = id
        self:replayAnimation()
        
        looping = loop == nil and true or loop
        
        -- Re-centering
        _, _, self.ox, self.oy = kit.getDimensions(id)
      end
    end
    
    function entity:updateAnimation(dt)
      local max_frames = #animations[current_animation].frames
      local frame_rate = animations[current_animation].frame_rate

      -- Stop from playing if max frame is reached and looping is false
      self.playing = not (looping == false and current_frame >= max_frames)
      
      -- Move to next frame (or go back to one)
      if max_frames > 1 and self.playing then
        frame_timer = frame_timer + dt
        if frame_timer >= frame_rate then
          frame_timer = frame_timer - frame_rate
          current_frame = current_frame >= max_frames and 1 or current_frame + 1
        end
      end
    end
    
    function entity:draw()
      love.graphics.draw(image, animations[current_animation].frames[current_frame],
        self.x, self.y, self.r, self.sx, self.sy, self.ox, self.oy
      )
    end
    

    
    return entity
  end
  
  return kit
end
