-- TEXT MENU

function newMenu(font, button_padding, vars)
  -- Local Vars are usually things that I don't want to access outside of the Menu
  local bfont = font or love.graphics.newFont()
  local bpadding = button_padding or 2-- How far the boxlines are to each button's text
  local tcolor = vars and vars.textcolor or {0, 0, 0}
  local bcolor = vars and vars.buttoncolor or {200/255, 200/255, 200/255}
  local scolor = vars and vars.selectedcolor or {200/255, 200/255, 0}
  local rcolor = vars and vars.registeredcolor or {1, 1, 0}
  local ocolor = vars and vars.bordercolor or {1, 1, 1}
  local rx     = vars and vars.roundx or 0
  local ry     = vars and vars.roundy or 0
  
  local menu ={             -- Ingredients of a Menu
    buttons = {},           -- A list of buttons / items
    selected_index = nil,   -- For drawing
    registered_index = nil, -- So that we can cancel what button was pressed
    active = true           -- Usually used for submenus
  }
  
  function menu:addButton(text, x, y, fn) -- fn = function
    table.insert(self.buttons, {
      text = text,
      x = x,
      y = y,
      width = 0,
      height = bfont:getHeight(text),
      fn = fn or function() end,
      active = true
      }
    )
  
  function menu:clear()
    self.buttons = {}
    self.selected_index = nil
    self.active = true
  end

    -- The following procedures center the image to the inputted x and y... renewing their x and y altogether.
    -- (I just prefer using x and y at the center of the buttons.)

    -- Gets the width of the longest button by default and resizes each menu by that, unless you set it later on
    local width = bfont:getWidth(text)
    for i = 1, #self.buttons do
      if self.buttons[i].width > width then
        width = self.buttons[i].width
      end
    end
      
    for i = 1, #self.buttons do
      self.buttons[i].width = width
    end
  end
   
  -- Cycling through menu items (Pressing Up/Down Left/Right or Mousewheel events are preferred)
  function menu:cycleUp()
    if self.active then
      -- Look for an active button before this index
      local active_finder = self.selected_index == nil and 1 or self.selected_index - 1
      while active_finder > 0 and not self.buttons[active_finder].active do
        active_finder = active_finder - 1
      end
      
      -- If no active is found, start from the last index and loop again
      if active_finder == 0 then
        active_finder = #self.buttons
        while active_finder > 0 and not self.buttons[active_finder].active do
          active_finder = active_finder - 1
        end
      end
      
      -- Assign selected_index to active button, nil if no active buttons are detected
      self.selected_index = active_finder == 0 and nil or active_finder
    end
  end
   
  function menu:cycleDown()
    if self.active then
      -- Look for an active button after this index
      local active_finder = self.selected_index == nil and 1 or self.selected_index + 1
      while active_finder <= #self.buttons and not self.buttons[active_finder].active do
        active_finder = active_finder + 1
      end
      
      -- If no active is found, start from the beginning with the same loop
      if active_finder > #self.buttons then
        active_finder = 1
        while active_finder <= #self.buttons and not self.buttons[active_finder].active do
          active_finder = active_finder + 1
        end
      end
      
      -- Assign selected_index to active button, nil if no active buttons are detected
      self.selected_index = active_finder > #self.buttons and nil or active_finder
    end
  end
   
  -- Detecting where the mouse is currently at (preferrably only under love.mousemoved(x, y)
  function menu:mouseOver(x, y)
    if self.active then
      self.selected_index = nil
      for i, v in pairs(self.buttons) do
        if v.active then
          if v.x + (v.width / 2) + bpadding > x and x > v.x - (v.width / 2) - bpadding then
            if v.y + (v.height / 2) + bpadding > y and y > v.y - (v.height / 2) - bpadding then
              self.selected_index = i
            end
          end
        end
      end
    end
  end
   
  -- Pushing and Releasing Buttons: Vital for Cancelling what you have accidentally pressed
  function menu:press()
    if self.active then self.registered_index = self.selected_index end
  end
   
  function menu:release()
    if self.active then
      local registered_index = self.registered_index
      self.registered_index = nil
        
      --menu.selected_index must NOT be nil for this to activate
      if self.selected_index and self.selected_index == registered_index then
        self.buttons[self.selected_index].fn()
      end
    end
  end

  function menu:draw()
    if self.active then
      love.graphics.setFont(bfont)
      for i, v in pairs(self.buttons) do
        -- I like using x as the center of the image instead of its true x
        local x = v.x - (v.width / 2)
        local y = v.y - (v.height / 2)
        local width = v.width + (bpadding * 2)
        local height = v.height + (bpadding * 2)
        
        -- Button
        if i == self.registered_index then
          love.graphics.setColor(rcolor)
        elseif i == self.selected_index then
          love.graphics.setColor(scolor)
        else
          love.graphics.setColor(bcolor)
        end
        love.graphics.rectangle('fill', x - bpadding, y - bpadding, width, height, rx, ry)
        
        -- Borders
        love.graphics.setColor(ocolor)
        love.graphics.rectangle('line', x - bpadding, y - bpadding, width, height, rx, ry)

        -- Text
        x = v.x - (bfont:getWidth(v.text) / 2)
        y = v.y - (bfont:getHeight(v.text) / 2)
        
        tcolor[4] = v.active and 1 or 0.3
        
        love.graphics.setColor(tcolor)
        love.graphics.print(v.text, x, y)
      end
    end
  end
  
  return menu
end

-- SPRITE MENU

function newSpriteMenu(image)
  local menu = {
    buttons = {},
    selected_index = nil,
    registered_index = nil,
    active = true
  }
  
  function menu:addButton(qx, qy, qw, qh, x, y, fn, animate)
    local quad = love.graphics.newQuad(qx, qy, qw, qh, image:getWidth(), image:getHeight())
    table.insert(
      self.buttons,
      {
        quad = quad,
        x = x,
        y = y,
        width = qw,
        height = qh,
        ox = qw / 2,
        oy = qh / 2,
        fn = fn or function() end
      }
    )
    
    if animate then -- To animate while pressing button quad MUST be on the RIGHT of the original quad
      local quad2 = love.graphics.newQuad(qx + qw, qy, qw, qh, image:getWidth(), image:getHeight())
      menu.buttons[#menu.buttons].quad2 = quad2
    end
  end

  -- Detecting where the mouse is currently at (preferrably only under love.mousemoved(x, y)
  function menu.mouse_over(x, y)
    if menu.active then
      menu.selected_index = nil
      for i, v in pairs(menu.buttons) do
        if x >= v.x - (v.width / 2) and x <= v.x + (v.width / 2) then
          if y >= v.y - (v.width / 2) and y <= v.y + (v.width / 2) then
            menu.selected_index = i
          end
        end
      end
    end
  end

  -- Pushing and Releasing Buttons: Vital for Cancelling what you have accidentally pressed
  function menu.button_press()
    if menu.active then menu.registered_index = menu.selected_index end
  end
   
  function menu.button_release()
    if menu.active then
      local registered_index = menu.registered_index
      menu.registered_index = nil
        
      --menu.selected_index must NOT be nil for this to activate
      if menu.selected_index and menu.selected_index == registered_index then
        menu.buttons[menu.selected_index].fn()
      end
    end
  end

  function menu.draw()
    if menu.active then
      love.graphics.setColor(1,1,1)
      for i, v in pairs(menu.buttons) do
        if i == menu.registered_index and love.mouse.isDown(1) and v.quad2 then
          love.graphics.draw(image, v.quad2, v.x, v.y, 0, 1, 1, v.ox, v.oy)
        else
          love.graphics.draw(image, v.quad, v.x, v.y, 0, 1, 1, v.ox, v.oy)
        end
      end
    end
  end
  
  return menu
end
