-- My TileSet Manager
-- It's one of the practical uses of SpriteBatch

function newTileManager(image, tile_width, tile_height, maxsprites)
  local tile_manager = {}
  local tile_chars = {} -- A dictionary of tile chars
  local tile_batch = love.graphics.newSpriteBatch(image, maxsprites or 1)
  local map = {}
  
  -- This gives each of the quads in our image an 'id'
  -- Each id will represent one-character strings (at least that's the intention)
  -- Since we already pre-configured each tile's size,
  --  we only need the x and y coordinates within the main image
  function tile_manager:addChar(id, x, y, level)
    tile_chars[id] = {
      quad = love.graphics.newQuad(x, y, tile_width, tile_height, image:getWidth(), image:getHeight()),
      level = level -- Usually: 0 = Air, 1 = Solid, 2 = Etc.
    }
  end
   
  -- If you're too lazy to addChar() one by one
  function tile_manager:setDictionary(ids)
    local id_table = {} -- This is for those who want randomized ids
      
    for row = 1, #ids do
      for col = 1, #ids[row] do
        local x = (col - 1) * (tile_width  + 1) -- (tile_width + 1) prevents texture bleeding
        local y = (row - 1) * (tile_height + 1)
        local c = ids[row]:sub(col, col)
        local l = row - 1
            
        table.insert(id_table, c)
        self:addChar(c, x, y, l)
      end
    end

    return id_table
  end
  
  function tile_manager:loadTileMap(tile_map)
    map = tile_map
  end
  
  -- Getting a tile's properties from the map
  function tile_manager:getTile(col, row)
    local x = (col-1) * tile_width
    local y = (row-1) * tile_height
    local level = 0
    
    -- If tile is not outside the map
    if not (row < 1 or row > #map or col < 1 or col > map[row]:len()) then
      local char = map[row]:sub(col, col)
      level = tile_chars[char].level
    end
    return level, x, y
  end

  -- It's advisable to update only when x, y, or even map has changed.
  -- Otherwise, it's counter-productive to take advantage of spritebatch
  function tile_manager:update(nx, ny)
    tile_batch:clear()  -- Clear SpriteBatch buffer
    
    start_x = nx or 0
    start_y = ny or 0

    for row = 1, #map do
      for col = 1, #map[row] do -- or map[row]:len()
        -- Just making the code readable at this point using local variables
        local tx = start_x + ((col - 1) * tile_width)
        local ty = start_y + ((row - 1) * tile_height)
        local tc = map[row]:sub(col, col) -- Getting 1 character in each string
            
        if tile_chars[tc] then -- Add a tile if it's a valid tile_type
          tile_batch:add(tile_chars[tc].quad, tx, ty)
        end
      end
    end

    -- Save data to graphics card
    -- (this is what makes SpriteBatch more memory-efficient than drawing an image over and over)
    tile_batch:flush()  
  end
   
  function tile_manager:draw()
    love.graphics.draw(tile_batch)
  end

  return tile_manager
end
