function newBoard(files, representation, goal_index, turn_counter, move_history, history_position)
  -- 1. Internal Values
  local board = {
    files = files,
    ranks = math.ceil(representation:len() / files),
    goal_index = goal_index,
    squares = {},
    players = {},
    moves = {},
    -- Move History
    turn_counter = turn_counter or 1,
    move_history = move_history or {},
  }
  board.history_position = history_position or #board.move_history
  
  -- 2. Putting each Character in the Squares Table
  for i = 1, representation:len() do
    local c = representation:sub(i, i)

    -- If the index is equal to goal_index, and that it's a rink notation
    if i == goal_index and c == notations.rink then
      table.insert(board.squares, notations.goal)
    else
      table.insert(board.squares, c)
    end
    
    -- Check if it's on the notations table
    local found = false
    for _, v in pairs(notations) do
      if v == c then found = true end
    end
    
    -- If not one of the notations, then it's a numerical value (a player)
    if not found then
      board.players[tonumber(c)] = i
    end
  end
  
  -- 3. Getter Methods
  function board:getFile(index)
    return ((index - 1) % self.files) + 1
  end
  function board:getRank(index)
    return math.ceil(index / self.files)
  end
  function board:getIndex(file, rank)
    return ((rank - 1) * self.files) + file
  end
  
  -- 4. Movements
  function board:movePiece(from, to, slide, overwrite)
    -- Overwrite History by default
    local overwrite_history = overwrite == nil and true or overwrite

    if overwrite_history and self.history_position < #self.move_history then
      -- Delete moves beyond the current history position
      repeat
        table.remove(self.move_history, self.history_position+1)
      until #self.move_history == self.history_position
    end
    
    if from ~= to then
      -- Squares
      self.squares[to] = self.squares[from]
      self.squares[from] = slide and (from == self.goal_index and notations.goal or notations.rink) or notations.slider

      -- Players
      for i = 1, #self.players do
        if self.players[i] == from then
          self.players[i] = to
        end
      end
    end

    -- Record Move
    table.insert(self.move_history, {from = from, to = to, slide = slide})
    self.history_position = self.history_position + 1
  end

  -- 5. Undo/Redo/Goto
  function board:undoMove()
    if self.history_position > 0 then
      local from  = self.move_history[self.history_position].to
      local to    = self.move_history[self.history_position].from
      local slide = self.move_history[self.history_position].slide
      
      -- Similar to board:movePiece(), but not the same
      if from ~= to then
        self.squares[to]   = self.squares[from]
        self.squares[from] = slide and (from == self.goal_index and notations.goal or notations.rank) or notations.slider
      end
      for i = 1, #self.players do
        if self.players[i] == from then
          self.players[i] = to
        end
      end

      self.history_position = self.history_position - 1
    end
  end
  function board:redoMove()
    if self.history_position < #self.move_history then
      self.history_position = self.history_position + 1
      local from = self.move_history[self.history_position].from
      local to = self.move_history[self.history_position].to
      local slide = self.move_history[self.history_position].slide
      self:movePiece(from, to, slide, false)
    end
  end
  function board:gotoMove(index)
    if self.history_position < index then
      repeat
        self:redoMove()
      until self.history_position == index
    elseif self.history_position > index then
      repeat
        self:undoMove()
      until self.history_position == index
    end
  end
  
  -- 6. Path Checker searches if the index can move to a sliding tile
  function board:checkSliders(index, checked_indexes)
    table.insert(checked_indexes, index)

    local test = index - 1      -- Adjacent Index
    local index_checked = false -- Checks if "test" is already in checked_indexes table
    local slide_found = false   -- Check if "test" has a sliding tile
    
    -- Left
    if self:getFile(index) > 1 then
      -- Checking if test is already in checked_indexes
      for _, v in pairs(checked_indexes) do
        if v == test then index_checked = true end
      end
      -- If it's a new index
      if index_checked == false then
        -- If it's another slider, check that for a sliding tile
        if self.squares[test] == notations.slider then
          slide_found = self:checkSliders(test, checked_indexes)
          
        -- If it's a rink or a goal, then we have found our sliding tile
        elseif self.squares[test] == notations.rink or self.squares[test] == notations.goal then
          slide_found = true
        end
      end
    end
    
    -- Up
    if slide_found == false and self:getRank(index) > 1 then
      test = index - files
      index_checked = false
      for _, v in pairs(checked_indexes) do
        if v == test then index_checked = true end
      end
      if index_checked == false then
        if self.squares[test] == notations.slider then
          slide_found = self:checkSliders(test, checked_indexes)
        elseif self.squares[test] == notations.rink or self.squares[test] == notations.goal then
          slide_found = true
        end
      end
    end
    
    -- Right
    if slide_found == false and self:getFile(index) < files then
      test = index + 1
      index_checked = false
      for _, v in pairs(checked_indexes) do
        if v == test then checked = true end
      end
      if index_checked == false and self.squares[test] == notations.slider then
        slide_found = self:checkSliders(test, checked_indexes)
      elseif self.squares[test] == notations.rink or self.squares[test] == notations.goal then
        slide_found = true
      end
    end
    
    -- Down
    if slide_found == false and self:getRank(index) < self.ranks then
      test = index + files
      index_checked = false
      for _, v in pairs(checked_indexes) do
        if v == test then checked = true end
      end
      if index_checked == false then
        if self.squares[test] == notations.slider then
          slide_found = self:checkSliders(test, checked_indexes)
        elseif self.squares[test] == notations.rink or self.squares[test] == notations.goal then
          slide_found = true
        end
      end
    end
    
    return slide_found
  end
  
  -- 7. Move Generator
  function board:generateMoves(i)
    local moves = {
      ['up']    = {index = 0, slide = false},
      ['down']  = {index = 0, slide = false},
      ['left']  = {index = 0, slide = false},
      ['right'] = {index = 0, slide = false}
    }

    local squares = self.squares
    local t
    
    -- Up
    if self:getRank(i) > 1 then
      t = i - files
      if squares[t] == notations.slider then
        moves['up'].index = t
      elseif squares[t] == notations.rink or squares[t] == notations.goal then
        while self:getRank(t) > 1 and (squares[t-files] == notations.rink or squares[t-files] == notations.goal) do
          t = t-files
        end
        moves['up'].index = t
        moves['up'].slide = true
      end
    end
    -- Down
    if self:getRank(i) < self.ranks then
      t = i + files
      if squares[t] == notations.slider then
        moves['down'].index = t
      elseif squares[t] == notations.rink or squares[t] == notations.goal then
        while self:getRank(t) < self.ranks and (squares[t+files] == notations.rink or squares[t+files] == notations.goal) do
          t = t+files
        end
        moves['down'].index = t
        moves['down'].slide = true
      end
    end

    -- Right
    if self:getFile(i) < self.files then
      t = i + 1
      if squares[t] == notations.slider then
        moves['right'].index = t
      elseif squares[t] == notations.rink or squares[t] == notations.goal then
        while self:getFile(t) < files and (squares[t+1] == notations.rink or squares[t+1] == notations.goal) do
          t = t + 1
        end
        moves['right'].index = t
        moves['right'].slide = true
      end
    end
    
    -- Left
    if self:getFile(i) > 1 then
      t = i - 1
      if squares[t] == notations.slider then
        moves['left'].index = t
      elseif squares[t] == notations.rink or squares[t] == notations.goal then
        while self:getFile(t) > 1 and (squares[t-1] == notations.rink or squares[t-1] == notations.goal) do
          t = t - 1
        end
        moves['left'].index = t
        moves['left'].slide = true
      end
    end
    
    return moves
  end
  
  function board:getRepresentation()
    local representation = ''
    for i = 1, #self.squares do
      representation = representation .. self.squares[i]
    end
    return representation
  end
  
  function board:evaluateWinner()
    local winner = 0
    for i, v in pairs(self.players) do
      if v == self.goal_index then
        winner = i
      end
    end
    return winner
  end
  
  return board
end