local TileSet = require 'lib/tilemanager'
local tile_width, tile_height = 32, 30

local bump_sound = love.audio.newSource('sfx/bump.oga', 'static')
local gameover_sound = love.audio.newSource('sfx/gameover.wav', 'static')

function newGuiBoard(files, representation, goal_index, turn_counter, move_history, history_position)
  local gb = newBoard(files, representation, goal_index, turn_counter, move_history, history_position)

  -- GUI ELEMENTS --
  local img = love.graphics.newImage('img/board.png')
  
  -- Rink
  rink = newTileManager(img, tile_width, tile_height, gb.files * gb.ranks)
  rink:setDictionary({'GR'})
  
  -- Slider
  sk = newAnimatedEntityKit(img)
  sk.newAnimation('idle',  {0, 31, tile_width, tile_height, 1}, 1  )
  sk.newAnimation('ready', {0, 62, tile_width, tile_height, 4}, 0.2)
  sk.newAnimation('pass',  {0, 127, 28, 8, 9}, 0.3)
  sk.slideSound = love.audio.newSource('sfx/slide.flac', 'static')

  local function newSlider(x, y, i)
    local slider = sk.spawn(x, y, tile_width, tile_height, 'idle')
    slider.index = i

    function slider.slide()
      sk.slideSound:stop()
      sk.slideSound:play()
    end

    return slider
  end
  
  local pass = sk.spawn(0, 0, 1, 1, 'pass')
  gb.can_slide = true
  
  -- Player
  local function addPlayer(x, y, v, i)
    local player = newPlayer(x, y, v)
    player.index = i
    player.ai = false
    return player
  end
  
  -- Moves
  mk = newStaticEntityKit(img)
  mk.newQuad('rink',    0, 92, tile_width, tile_height)
  mk.newQuad('slider', 33, 92, tile_width, tile_height)

  local function newMove(x, y, i, slide)
    local move = mk.spawn(x, y, slide == true and 'rink' or 'slider')
    move.index = i
    return move
  end
  
  local moves = {}

  -- Rink
  local new_rep = {}
  for r = 1, gb.ranks do
    local rank = ''
    for c = 1, gb.files do
      local s = ((r-1) * gb.files) + c
      
      if gb:getIndex(c,r) == gb.goal_index then
        rank = rank .. notations.goal
      elseif gb.squares[s] == notations.void then
        rank = rank .. notations.void
      else
        rank = rank .. notations.rink
      end
    end
    table.insert(new_rep, rank)
  end
  rink:loadTileMap(new_rep)
  rink:update()
  
  -- Sliders & Players
  gb.sliders = {}
  gb.players = {}
  for i, v in ipairs(gb.squares) do
    local x = (gb:getFile(i) * tile_width) - (tile_width / 2)
    local y = (gb:getRank(i) * tile_height) - (tile_height / 2)

    if v == notations.slider then -- Slider Only
      table.insert(gb.sliders, newSlider(x, y, i))
    else -- Slider + Player
      local is_a_notation = false
      for _, n in pairs(notations) do
        if n == v then
          is_a_notation = true
        end
      end
      
      if not is_a_notation then
        turn = tonumber(v)
        gb.players[turn] = addPlayer(x, y, turn, i)
        table.insert(gb.sliders, newSlider(x, y, i))
      end
    end
  end
  
  -- GUI Board Update
  gb.updating = true
  local move_speed = 225

  function gb:update(dt)
    if self.updating then
      for _, v in pairs(self.sliders) do v:updateAnimation(dt) end
      for _, v in pairs(self.players) do v:update(dt) end

      if self.draw_moves then
        local m = moves['up']
        if m.index > 0 and m.y > m.target_y then
          m.y = math.max(m.y - move_speed * dt, m.target_y)
        end
        m = moves['down']
        if m.index > 0 and m.y < m.target_y then
          m.y = math.min(m.y + move_speed * dt, m.target_y)
        end
        m = moves['left']
        if m.index > 0 and m.x > m.target_x then
          m.x = math.max(m.x - move_speed * dt, m.target_x)
        end
        m = moves['right']
        if m.index > 0 then
          m.x = math.min(m.x + move_speed * dt, m.target_x)
        end
      end
      
      if not gb.can_slide then
        pass:setPosition(self.players[board.turn_counter].x, self.players[board.turn_counter].y)
        pass:updateAnimation(dt)
      end
    end
  end
  
  -- GUI Board Draw
  local boardwidth = screen.width / (tile_width * gb.files)
  local boardheight = screen.height / (tile_height * gb.ranks)
  local boardscale = boardheight < screen.height * boardwidth and boardheight or boardwidth
  local boardx = screen.width /boardscale/2 - (tile_width  * gb.files)/2
  local boardy = screen.height/boardscale/2 - (tile_height * gb.ranks)/2 

  function gb:draw()
    love.graphics.push()
    love.graphics.scale(boardscale)
    love.graphics.translate(boardx, boardy)      
      rink.draw()
      for _, v in pairs(self.sliders) do v:draw() end
      for _, v in pairs(self.players) do v:draw() end

      if self.draw_moves then
        for _, v in pairs(moves) do
          if v.index > 0 then v:draw() end
        end
      end
      if not gb.can_slide then pass:draw() end
    love.graphics.pop()
  end
  
  -- Other GUI Functions
  function gb:loadMoves()
    moves = {}
    self.draw_moves = true
    local p = self.players[self.turn_counter]

    for k, v in pairs(self.moves) do
      local i = v.index
      local x = (self:getFile(i) * tile_width) - (tile_width / 2)
      local y = (self:getRank(i) * tile_height) - (tile_width / 2)

      local move = newMove(p.x, p.y, i, v.slide)
      
      move.target_x = x
      move.target_y = y
      
      moves[k] = move
    end
    
    for _, v in pairs(self.sliders) do
      if v.index == p.index then
        v:setAnimation('ready')
      end
    end
    self.players[self.turn_counter]:setAnimation('ready')
  end
  
  function gb:movePiece(from, to, slide, overwrite)
    local moving_slider, moving_player = 0, 0

    -- Overwrite History by default
    local overwrite_history = overwrite == nil and true or overwrite

    if overwrite_history then
      -- Delete moves beyond the current history position
      repeat
        table.remove(self.move_history, self.history_position+1)
      until #self.move_history == self.history_position
    end
    
    if from ~= to then
      -- Squares
      self.squares[to] = self.squares[from]
      self.squares[from] = slide and (from == self.goal_index and notations.goal or notations.rink) or notations.slider
    end
    
    -- Sliders
    if slide then
      for i, v in ipairs(self.sliders) do
        if v.index == from then
          v.index = to
          moving_slider = i
        end
      end
    end
    
    -- Players
    for i, v in ipairs(self.players) do
      if v.index == from then
        v.index = to
        moving_player = i
        v.target_x = (self:getFile(to) * tile_width) - (tile_width / 2)
        v.target_y = (self:getRank(to) * tile_height) - (tile_height / 2)
      end
    end

    -- Record Move
    if overwrite_history then
      table.insert(self.move_history, {from = from, to = to, slide = slide})
    end
    self.history_position = self.history_position + 1
    
    if moving_slider > 0 then board.sliders[moving_slider].slide() end

    return moving_slider, moving_player
  end

  function gb:undoMove()
    local moving_slider, moving_player = 0, 0
    if self.history_position > 0 then
      local from  = self.move_history[self.history_position].to
      local to    = self.move_history[self.history_position].from
      local slide = self.move_history[self.history_position].slide
      if from ~= to then
        self.squares[to]   = self.squares[from]
        self.squares[from] = slide and (from == self.goal_index and notations.goal or notations.rink) or notations.slider
      end
      
      if slide then
        for i, v in ipairs(self.sliders) do
          if v.index == from then
            v.index = to
            moving_slider = i
          end
        end
      end

      for i, v in ipairs(self.players) do
        if v.index == from then
          v.index = to
          moving_player = i
          
          v.target_x = (self:getFile(to) * tile_width) - (tile_width / 2)
          v.target_y = (self:getRank(to) * tile_height) - (tile_height / 2)
        end
      end

      self.history_position = self.history_position - 1
    end
    return moving_slider, moving_player
  end

  function gb:redoMove()
    local moving_slider, moving_player, direction = 0, 0
    if self.history_position < #self.move_history then
      local histp = self.history_position + 1
      local from  = self.move_history[histp].from
      local to    = self.move_history[histp].to
      local slide = self.move_history[histp].slide
      moving_slider, moving_player = self:movePiece(from, to, slide, false)
    end
    
    return moving_slider, moving_player
  end

  function gb:evaluateWinner()
    local winner = 0
    for i, v in pairs(self.players) do
      if v.index == self.goal_index then
        winner = i
      end
    end
    return winner
  end
  
  function gb:bump()
    bump_sound:stop()
    bump_sound:play()
  end
  
  function gb:win()
    gameover_sound:stop()
    gameover_sound:play()
  end

  return gb
end