local pseudobuttons = require 'src/objects/pseudobuttons'

state_load = newState()

local games, message, selected_index, left, right

local function updateGuiElements()
  if #games > 0 then
    -- Initialize Board
    local b = games[selected_index]
    board = newGuiBoard(b.files, b.representation, b.goal_index, b.turn_counter, b.move_history, b.history_position)
    for i = 1, #board.players do
      board.players[i].ai = (b.players[i] == 1)
    end
    -- Message
    message = 'Game ' .. selected_index
    
    -- Activate Menu Buttons
    menu.buttons[1].active = true
    menu.buttons[2].active = true
  else
    -- Default Display (Empty) Board
    board = newGuiBoard(3,'RRRRRRRRR',5)
    
    -- Message
    message = 'No Saved Games'
    
    -- Deactivate Menu Buttons
    menu.buttons[1].active = false
    menu.buttons[2].active = false
    menu:cycleDown()
  end
end

local function deleteGame()
  if #games > 0 then
    -- Remove File
    love.filesystem.remove(games[selected_index].filename)

    -- Remove from Table
    table.remove(games, selected_index)
    
    -- Adjust Selected Index
    selected_index = selected_index <= #games and selected_index or #games
    
    updateGuiElements()
  end
end

function state_load.load()
  -- Default Sandbox Folder
  local folder_items = love.filesystem.getDirectoryItems('')

  -- List Down only files that end in ".game"
  local list = {}
  for _, file in pairs(folder_items) do
    if file:sub(-4) == '.gam' then
      table.insert(list, file)
    end
  end
  
  -- Fill our "games" table with the necessary stuff from the "list" of files
  games = {}
  
  for _, gamfile in pairs(list) do
    -- 1. For each ".gam file", insert all it's "lines" in "data"
    local data = {}
    for line in love.filesystem.lines(gamfile) do
      table.insert(data, line)
    end
    
    -- 2. Data Extraction
    local files = tonumber(data[1])
    local representation = data[2]
    local goal_index = tonumber(data[3])
    local turn_counter = tonumber(data[4])
    local history_position = tonumber(data[5])
    local players = {}
    for word in data[6]:gmatch('%S+') do
      table.insert(players, tonumber(word))
    end
    
    -- 3. Extracting Move History from data[6] and above:
    local move_history = {}
    for i = 7, #data do
      -- a. List of 'words' in each 'data'
      local words = {}
      -- '%S+' = non-space characters; gmatch then splits the words that have space in them.
      for word in data[i]:gmatch('%S+') do
        table.insert(words, word)
      end
      
      -- b. Inserting the separated 'words' into move_history and converting them at the same time
      table.insert(move_history, {
        from = tonumber(words[1]),
        to = tonumber(words[2]),
        slide = words[3] == 'true' -- and true or false
      })
    end
    
    -- 4. Inserting all of that into the 'games' list
    table.insert(games, {
      filename = gamfile,
      files = files,
      representation = representation,
      goal_index = goal_index,
      turn_counter = turn_counter,
      move_history = move_history,
      history_position = history_position,
      players = players
    })
  end

  -- Selected Index
  selected_index = #games > 0 and 1 or 0
  
  -- Display Pseudo Buttons
  left, right = newPseudoButtons()
  menu:clear()
  menu:addButton('PLAY',   screen.width / 2, screen.height / 2 + 22, function() state_machine.change(state_play)  end)
  menu:addButton('DELETE', screen.width / 2, screen.height / 2 + 44, function() deleteGame() end)
  menu:addButton('BACK',   screen.width / 2, screen.height / 2 + 66, function() state_machine.change(state_title, 2) end)
  updateGuiElements()
  menu:cycleDown()
end

function state_load.update(dt)
  left:setAnimation(love.keyboard.isDown('left') and 'leftdown' or 'leftup')
  right:setAnimation(love.keyboard.isDown('right') and 'rightdown' or 'rightup')
end

function state_load.draw()
  love.graphics.printf(message, 0, 20, screen.width, 'center')
  left:draw()
  right:draw()
  menu:draw()
end

function state_load.keypressed(key)
  if key == 'escape' then
    state_machine.change(state_title, 2)
  elseif key == 'return' or key == 'space' or key == 'kpenter' then
    menu:press()
  elseif key == 'up' then
    menu:cycleUp()
  elseif key == 'down' then
    menu:cycleDown()
  elseif #games > 0 then
    if key == 'left' then
      selected_index = selected_index == 1 and #games or selected_index - 1
    elseif key == 'right' then
      selected_index = selected_index == #games and 1 or selected_index + 1
    end
    updateGuiElements()
  end
end

function state_load.keyreleased(key)
  menu:release()
end